@@ -1,4 +1,4 @@
-# Copyright 2021 Universität Tübingen, DKFZ and EMBL
+# Copyright 2021 - 2022 Universität Tübingen, DKFZ and EMBL
 # for the German Human Genome-Phenome Archive (GHGA)
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -26,7 +26,7 @@ import botocore.client
 import botocore.config
 import botocore.configloader
 import botocore.exceptions
-from pydantic import BaseSettings
+from pydantic import BaseSettings, Field
 
 from .object_storage_dao import (
     BucketAlreadyExists,
@@ -37,11 +37,11 @@ from .object_storage_dao import (
     ObjectNotFoundError,
     ObjectStorageDao,
     ObjectStorageDaoError,
-    OutOfContextError,
     PresignedPostURL,
     validate_bucket_id,
     validate_object_id,
 )
+from .utils import OutOfContextError
 
 
 class S3ConfigBase(BaseSettings):
@@ -67,11 +67,43 @@ class S3ConfigBase(BaseSettings):
             Defaults to None.
     """
 
-    s3_endpoint_url: str
-    s3_access_key_id: str
-    s3_secret_access_key: str
-    s3_session_token: Optional[str] = None
-    aws_config_ini: Optional[Path] = None
+    s3_endpoint_url: str = Field(
+        ..., example="http://localhost:4566", description="URL to the S3 API."
+    )
+    s3_access_key_id: str = Field(
+        ...,
+        example="my-access-key-id",
+        description=(
+            "Part of credentials for login into the S3 service. See:"
+            + " https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html"
+        ),
+    )
+    s3_secret_access_key: str = Field(
+        ...,
+        example="my-secret-access-key",
+        description=(
+            "Part of credentials for login into the S3 service. See:"
+            + " https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html"
+        ),
+    )
+    s3_session_token: Optional[str] = Field(
+        None,
+        example="my-session-token",
+        description=(
+            "Part of credentials for login into the S3 service. See:"
+            + " https://boto3.amazonaws.com/v1/documentation/api/latest/guide/credentials.html"
+        ),
+    )
+    aws_config_ini: Optional[Path] = Field(
+        None,
+        example="~/.aws/config",
+        description=(
+            "Path to a config file for specifying more advanced S3 parameters."
+            + " This should follow the format described here:"
+            # pylint: disable=line-too-long
+            + " https://boto3.amazonaws.com/v1/documentation/api/latest/guide/configuration.html#using-a-configuration-file"
+        ),
+    )
 
 
 @lru_cache
@@ -144,8 +176,6 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
                 - ObjectAlreadyExistsError
     """
 
-    _out_of_context_error = OutOfContextError(context_manager_name="ObjectStorageS3")
-
     def __init__(  # pylint: disable=too-many-arguments
         self,
         config: S3ConfigBase,
@@ -211,7 +241,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         Return `True` if it exists and `False` otherwise.
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(bucket_id)
 
@@ -246,7 +276,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         specified unique ID.
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(bucket_id)
 
@@ -265,7 +295,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         not empty.
         """
         if not isinstance(self._resource, boto3.resources.factory.ServiceResource):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(bucket_id)
 
@@ -288,7 +318,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         Return `True` if checks succeed and `False` otherwise.
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         if object_md5sum is not None:
             raise NotImplementedError("Md5 checking is not yet implemented.")
@@ -341,7 +371,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         a maximum size (bytes) for uploads (`max_upload_size`).
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(bucket_id)
         validate_object_id(object_id)
@@ -381,7 +411,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         You may also specify a custom expiry duration in seconds (`expires_after`).
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(bucket_id)
         validate_object_id(object_id)
@@ -412,7 +442,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         another bucket (`dest_bucket_id` and `dest_object_id`).
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(source_bucket_id)
         validate_object_id(source_object_id)
@@ -447,7 +477,7 @@ class ObjectStorageS3(ObjectStorageDao):  # pylint: disable=too-many-instance-at
         You may also specify a custom expiry duration in seconds (`expires_after`).
         """
         if not isinstance(self._client, botocore.client.BaseClient):
-            raise self._out_of_context_error
+            raise OutOfContextError()
 
         validate_bucket_id(bucket_id)
         validate_object_id(object_id)
